"""Gene panel specific constants"""

GENOME_BUILD_37 = "37"
GENOME_BUILD_38 = "GRCh38"

MASTER_LIST = (
    "BRAIN",
    "Cardiology",
    "CH",
    "CTD",
    "DIAB",
    "ENDO",
    "EP",
    "IBMFS",
    "IEM",
    "IF",
    "mcarta",
    "MHT",
    "MIT",
    "MOVE",
    "mtDNA",
    "NBS-M",
    "NEURODEG",
    "NMD",
    "PEDHEP",
    "PID",
    "PIDCAD",
    "OMIM-AUTO",
    "SKD",
)
